"use client";
import { jsx as _jsx } from "react/jsx-runtime";
import React from "react";
const StepIndexContext = React.createContext([0, () => { }]);
export function ScrollyRoot({ className, children, }) {
    const [selectedIndex, setSelectedIndex] = React.useState(0);
    return (_jsx(Scroller, { onStepChange: setSelectedIndex, triggerPosition: "40%", children: _jsx(StepIndexContext.Provider, { value: [selectedIndex, setSelectedIndex], children: _jsx("div", { className: className, "data-ch-selected-index": selectedIndex, children: children }) }) }));
}
export function ScrollyStep({ children, className, stepIndex, }) {
    const [selectedIndex, setSelectedIndex] = React.useContext(StepIndexContext);
    return (_jsx(ScrollerStep, { index: stepIndex, onClick: () => setSelectedIndex(stepIndex), className: className, "data-ch-selected": selectedIndex === stepIndex ? true : undefined, children: children }, stepIndex));
}
export function ScrollySticker({ stickers, className, }) {
    const [selectedIndex] = React.useContext(StepIndexContext);
    return _jsx("div", { className: className, children: stickers[selectedIndex] });
}
const ObserverContext = React.createContext(undefined);
const useLayoutEffect = typeof window !== "undefined" ? React.useLayoutEffect : React.useEffect;
function Scroller({ onStepChange, children, getRootMargin = defaultRootMargin, triggerPosition, debug = false, }) {
    const [observer, setObserver] = React.useState();
    const vh = useWindowHeight();
    useLayoutEffect(() => {
        const windowHeight = vh || 0;
        const handleIntersect = (entries) => {
            // if (debug || (window as any).chDebugScroller) {
            //   debugEntries(entries)
            // }
            entries.forEach((entry) => {
                if (entry.intersectionRatio > 0) {
                    const stepElement = entry.target;
                    onStepChange(+stepElement.stepIndex);
                }
            });
        };
        const observer = newIntersectionObserver(handleIntersect, getRootMargin(windowHeight, triggerPosition));
        setObserver(observer);
        return () => observer.disconnect();
    }, [vh]);
    return (_jsx(ObserverContext.Provider, { value: observer, children: children }));
}
function ScrollerStep({ as = "div", index, ...props }) {
    const ref = React.useRef(null);
    const observer = React.useContext(ObserverContext);
    useLayoutEffect(() => {
        if (observer) {
            observer.observe(ref.current);
        }
        return () => observer && observer.unobserve(ref.current);
    }, [observer]);
    useLayoutEffect(() => {
        const stepElement = ref.current;
        stepElement.stepIndex = index;
    }, [index]);
    return React.createElement(as, { ...props, ref });
}
function newIntersectionObserver(handleIntersect, rootMargin) {
    return new IntersectionObserver(handleIntersect, {
        rootMargin,
        threshold: 0.000001,
        root: null,
    });
}
function defaultRootMargin(vh, triggerPosition = "50%") {
    let y = vh * 0.5;
    if (triggerPosition.endsWith("%")) {
        const percent = parseFloat(triggerPosition.replace("%", ""));
        y = vh * (percent / 100);
    }
    else if (triggerPosition.endsWith("px")) {
        y = parseFloat(triggerPosition.replace("px", ""));
    }
    if (y < 0) {
        y = vh + y;
    }
    return `-${y - 2}px 0px -${vh - y - 2}px`;
}
function useWindowHeight() {
    const isClient = typeof window === "object";
    function getHeight() {
        return isClient ? document.documentElement.clientHeight : undefined;
    }
    const [windowHeight, setWindowHeight] = React.useState(getHeight);
    React.useEffect(() => {
        function handleResize() {
            setWindowHeight(getHeight());
        }
        window.addEventListener("resize", handleResize);
        return () => window.removeEventListener("resize", handleResize);
    }, []);
    useLayoutEffect(() => {
        // FIX when a horizontal scrollbar is added after the first layout
        setWindowHeight(getHeight());
    }, []);
    return windowHeight;
}
